Package: KinSwingR
Type: Package
Title: KinSwingR: network-based kinase activity prediction
Version: 1.28.0
Authors@R: person("Ashley J.", "Waardenberg", email =
    "a.waardenberg@gmail.com", role=c("aut", "cre"))
Description: KinSwingR integrates phosphosite data derived from mass-spectrometry
    data and kinase-substrate predictions to predict kinase activity. Several functions
    allow the user to build PWM models of kinase-subtrates, statistically infer PWM:substrate
    matches, and integrate these data to infer kinase activity.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5)
Imports: data.table, BiocParallel, sqldf, stats, grid, grDevices
biocViews: Proteomics, SequenceMatching, Network
RoxygenNote: 6.1.0
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/KinSwingR
git_branch: RELEASE_3_22
git_last_commit: 5de1986
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:42:26 UTC; biocbuild
Author: Ashley J. Waardenberg [aut, cre]
Maintainer: Ashley J. Waardenberg <a.waardenberg@gmail.com>

# Generated by roxygen2: do not edit by hand

export(buildPWM)
export(cleanAnnotation)
export(scoreSequences)
export(swing)
export(viewPWM)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(data.table,data.table)
importFrom(data.table,melt.data.table)
importFrom(grDevices,rgb)
importFrom(grid,dataViewport)
importFrom(grid,gpar)
importFrom(grid,grid.newpage)
importFrom(grid,grid.polygon)
importFrom(grid,grid.xaxis)
importFrom(grid,grid.yaxis)
importFrom(grid,plotViewport)
importFrom(grid,pushViewport)
importFrom(sqldf,sqldf)
importFrom(stats,sd)
importFrom(stats,setNames)

# KinSwingR

KinSwingR is an R package for inference of kinase activity from phosphoproteomics data.

## Installing KinSwingR

To obtain the latest version from github, install devtools in R and use the following:

```R
library(devtools)
devtools::install_github("awaardenberg/KinSwingR", build_vignettes = TRUE)
```

## Examples

To run KinSwingR, load the library and follow the examples in the vignette.

```R
library(KinSwingR)
vignette("KinSwingR")
```

## Contact

For more details, contact Ashley Waardenberg:
a.waardenberg@gmail.com

#' Swing statistic
#'
#' @description This function integrates the kinase-substrate predictions,
#' directionality of phosphopeptide fold change and signficance to assess
#' local connectivity (swing) of kinase-substrate networks. The final score
#' is a normalised and weighted score of predicted kinase activity. If 
#' permutations are selected, network node:edges are permutated. P-values will 
#' be calculated for both ends of the distribution of swing scores (positive and
#'  negative swing scores).
#' @param input_data A data.frame of phoshopeptide data. Must contain 4 columns
#'  and the following format must be adhered to. Column 1 - Annotation, Column 2
#'   - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
#'   - p-value [0-1]. This must be the same dataframe used in scoreSequences()
#' @param pwm_in List of PWMs created using buildPWM()
#' @param pwm_scores List of PWM-substrate scores created using
#' scoreSequences()
#' @param pseudo_count Pseudo-count acts at two levels. 1) It adds a small
#' number to the counts to avoid zero divisions, which also 2) avoids log-zero 
#' transformations. Note that this means that pos, neg and all values in the
#' output table include the addition of the pseudo-count. Default: "1"
#' @param p_cut_pwm Significance level for determining a significant
#' kinase-substrate enrichment. Default: "0.05"
#' @param p_cut_fc Significance level for determining a significant level of
#' Fold-change in the phosphoproteomics data. Default: "0.05"
#' @param permutations Number of permutations to perform. This will shuffle the 
#' kinase-subtrate edges of the network n times. To not perform permutations and
#'  only generate the scores, set permutations=1 or permutations=FALSE. Default:
#'   "1000"
#' @param return_network Option to return an interaction network for visualising
#' in cystoscape. Default = FALSE
#' @param verbose Turn verbosity on/off. To turn on, verbose=TRUE. Options are: 
#' "TRUE, FALSE". Default=FALSE
#'
#' @examples
#' ## import data
#' data(example_phosphoproteome)
#' data(phosphositeplus_human)
#'
#' ## clean up the annotations
#' ## sample 100 data points for demonstration
#' sample_data <- head(example_phosphoproteome, 100)
#' annotated_data <- cleanAnnotation(input_data = sample_data)
#'
#' ## build the PWM models:
#' set.seed(1234)
#' sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
#' 1000),]
#' pwms <- buildPWM(sample_pwm)
#'
#' ## score the PWM - substrate matches
#' ## Using a "random" background, to calculate the p-value of the matches
#' ## Using n = 100 for demonstration
#' ## set.seed for reproducibility
#' set.seed(1234)
#' substrate_scores <- scoreSequences(input_data = annotated_data,
#'                                    pwm_in = pwms,
#'                                    background = "random",
#'                                    n = 100)
#'
#' ## Use substrate_scores and annotated_data data to predict kinase activity.
#' ## This will permute the network node and edges 10 times for demonstration.
#' ## set.seed for reproducibility
#' set.seed(1234)
#' swing_output <- swing(input_data = annotated_data,
#'                       pwm_in = pwms,
#'                       pwm_scores = substrate_scores,
#'                       permutations = 10)
#'
#' @return A data.table of swing scores
#'
#' @export swing
#' @importFrom BiocParallel bplapply
#' @importFrom BiocParallel MulticoreParam
#' @importFrom stats setNames
#' @importFrom stats sd
#' @importFrom data.table melt.data.table

swing <-
  function(input_data = NULL,
           pwm_in = NULL,
           pwm_scores = NULL,
           pseudo_count = 1,
           p_cut_pwm = 0.05,
           p_cut_fc = 0.05,
           permutations = 1000,
           return_network = FALSE,
           verbose = FALSE) {
    #----------------------------------------------
    #format checks:
    if (is.null(input_data))
      stop("input_data not provided; you must provide an input table")
    if (!is.data.frame(input_data))
      stop("input_data is not a data.frame; you must provide an input table")
    if (is.null(pwm_in))
      stop(
        "pwm_in not provided; you must provide an input table containing
        computed position weight matrices using buildPWM()"
      )
    if (!is.list(pwm_in))
      stop(
        "pwm_in is not a list format; something has gone wrong. Make sure you
        compute the position weight matrices using buildPWM()"
      )
    if (is.null(pwm_scores))
      stop(
        "pwm_scores not provided; you must provide an input table containing
        computed scores using scoreSequences()"
      )
    if (!is.list(pwm_scores))
      stop(
        "pwm_scores is not a list format; something has gone wrong. Make sure
        PWM-substrate matches are scored using scoreSequences()"
      )
    if (p_cut_pwm >= 1)
      stop("p_cut_pwm needs to be less than 1; make sure your p-values are not
           log transformed")
    if (p_cut_fc >= 1)
      stop("p_cut_fc needs to be less than 1; make sure your p-values are not
           log transformed")
    if (permutations != FALSE &
        (!is.numeric(permutations) | permutations < 1))
      stop(
        "permutations needs to be a numerical number. Either 1 or FALSE (for no 
        permutation) or greater than 1 to set the number of pemutations"
      )
    
    #----------------------------------------------
    # 1. binarise p-values and fold change
    if (verbose) {
      start_time <- Sys.time()
      message("Start: ", start_time)
      message("[Step1/3] : Calculating Swing Scores")
    }
    pwm_pval <- pwm_scores$peptide_p
    # binarise p-values
    pwm_pval[, 3:ncol(pwm_pval)] <-
      ifelse(pwm_pval[, 3:ncol(pwm_pval)] > p_cut_pwm, 0, 1)
    # binarise FC
    input_data[, 3] <- ifelse(as.numeric(input_data[, 3]) > 0, 1, -1)
    # binarise p-value
    input_data[, 4] <-
      ifelse(as.numeric(input_data[, 4]) > p_cut_fc, 0, 1)
    # compute Sipk statistic (Sipk - see paper):
    input_data$Sipk <-
      as.numeric(input_data[, 3]) * as.numeric(input_data[, 4])
    
    # 2. merge tables, summarise counts and unique
    input_data$annotation <-
      paste(input_data$annotation, input_data$peptide, sep = "::")
    pwm_pval$annotation <-
      paste(pwm_pval$annotation, pwm_pval$peptide, sep = "::")
    data_merge <-
      unique(merge(input_data, pwm_pval, by = "annotation"))
    
    # 3. Swing scores of REAL data.
    swing_out <- swingScore(
      data_merge = data_merge,
      pwm_in = pwm_in,
      permute = FALSE,
      pseudo_count = pseudo_count
    )
    
    # 4. permute and calculate p-values
    if (permutations != FALSE && permutations > 1) {
      if (verbose) {
        message("[Step2/3] : Permuting Network")
      }
      # obtain permuted column names
      swing_names <- colnames(data_merge)[7:ncol(data_merge)]
      n_permute <- lapply(seq_len(permutations), function(i)
        sample(as.character(swing_names), length(swing_names),replace = FALSE))
      names(n_permute) <- paste("rand", seq_len(permutations), sep = "")
      
      # calculate swing scores (with permuted names) and return as vector
      swing_permute <- list(bplapply(seq_len(permutations), function(i)
        swingScore(
          data_merge = setNames(data.frame(data_merge),
                                c(colnames(data_merge)[1:6],
                                  n_permute[[i]])),
          pwm_in = pwm_in,
          permute = TRUE,
          pseudo_count = pseudo_count,
          n = i
        )))
      # returns ordered dataframe; order names and merge
      swing_permute_names <- swing_names[order(swing_names)]
      swing_permute <- data.frame("kinase" = swing_permute_names,
                                  swing_permute)
      colnames(swing_permute) <- c("kinase", names(n_permute))
      
      if (verbose) {
        message("[Step3/3] : Calculating p-values")
      }
      # obtain p-values, two sided
      swing_out$p_greater <-
        unlist(bplapply(seq_len(nrow(swing_out)), function(i)
          (sum(
            ifelse(
              as.numeric(swing_permute[swing_permute$kinase ==
                                         swing_out$kinase[i], ][2:ncol(swing_permute)]) >
                as.numeric(swing_out$swing_raw[i]), 1, 0), 
            na.rm = TRUE) + 1)
          / (as.numeric(permutations) + 1)))
      
      swing_out$p_less <-
        unlist(bplapply(seq_len(nrow(swing_out)), function(i)
          (sum(
            ifelse(
              as.numeric(swing_permute[swing_permute$kinase ==
                         swing_out$kinase[i], ][2:ncol(swing_permute)]) <
                         as.numeric(swing_out$swing_raw[i]), 1, 0
                        ), na.rm = TRUE) + 1)
                        / (as.numeric(permutations) + 1)))
      # order by p-value
      swing_out <- swing_out[order(swing_out$p_greater),]
    }
    
    if (verbose) {
      message("[FINISHED]\n")
      end_time <- Sys.time() - start_time
      message("Finish: ", Sys.time())
    }
    
    # set option to return a network
    network <- NULL
    if (return_network == "TRUE"){
      network <- data.table::melt(data_merge,
                                 id.vars = c("annotation"),
                                 measure.vars = 
                                 colnames(data_merge[7:ncol(data_merge)]))
      # keep "significant" edges
      network <- network[network$value == 1, ]
      network <- data.frame(
        "source" = as.character(network$variable),
        "target" = as.character(network$annotation)
      )
    }
    return(list("scores" = swing_out, "network" = network))
  }

# describeIn swing This helper function performs the swing score calculation
# no verbose in this helper
# permute - for setting right table format (using data.table) for merging.

swingScore <-
  function(data_merge,
           pwm_in,
           pseudo_count = 1,
           permute = FALSE,
           n = 1) {
    #----------------------------------------------
    data_merge <- data_merge$Sipk * data_merge[7:ncol(data_merge)]
    # propogation of NaN here due to small number division, add psuedo-count
    # count significant positive
    p_k <- colSums(data_merge == 1, na.rm = TRUE) + pseudo_count
    # count significant negative
    n_k <- colSums(data_merge == -1, na.rm = TRUE) + pseudo_count
    #all counts (both positive and negative).
    t_k <- p_k + n_k
    #----------------------------------------------
    # compute swing statistics
    pk_ratio <- (p_k)/(t_k)
    nk_ratio <- (n_k)/(t_k)
    swing_raw <- (pk_ratio) / (nk_ratio)
    p_n_all <-
      data.frame(
        "kinase" = colnames(data_merge),
        "pos" = p_k,
        "neg" = n_k,
        "all" = t_k,
        "pk" = pk_ratio,
        "nk" = nk_ratio,
        "swing_raw" = swing_raw
      )
    # include the count numbers:
    p_n_all <- merge(p_n_all, pwm_in$kinase, by = "kinase", sort = TRUE)
    # weighted by number of substrates
    # not possible to have zero - must have substrates to build model
    p_n_all$swing_raw <-
      log2(p_n_all$swing_raw) * log2(p_n_all$n)
    # weighted by size of kinase-substrate network:
    p_n_all$swing_raw <-
      p_n_all$swing_raw * log2(p_n_all$all)
    # z-score transform
    p_n_all$swing <-
      (p_n_all$swing_raw - mean(p_n_all$swing_raw, na.rm = TRUE)) / 
      sd(p_n_all$swing_raw, na.rm = TRUE)
    
    if (permute == "TRUE") {
      p_n_all <- p_n_all$swing_raw
    }
    return(p_n_all)
  }

#' Function for extracting peptide sequences from multimapped or complex 
#' annotated data
#'
#' @description This function extracts unique peptide:annotation combinations 
#' from complex annotated data and formats for further analysis using KinSwingR.
#' For instance, example input annotation may be: 
#' "A0A096MIX2|Ddx17|494|RSRYRTTSSANNPN". This function will extract the peptide
#'  sequence into a second column and associate it all annotations. See vignette
#'   for more details.
#' @param input_data A data.frame of phosphopeptide data. Must contain 4 columns
#'  and the following format must be adhered to. Column 1 - Annotation, Column 2
#'   - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
#'   - p-value [0-1]. This will extract the peptide sequences from Column1 and 
#'   replace all values in Column2 to be used in scoreSequences(). Where 
#'   peptide sequences have not been extracted from the annotation, leave 
#'   Column2 as NA's.
#' @param annotation_delimiter The character used to delimit annotations. 
#' Default="|"
#' @param multi_protein_delimiter The character used to delimit multi-protein 
#' assignments. Default=":". E.g. Ddx17:Ddx2
#' @param multi_site_delimiter The character used to delimit multi-site 
#' assignments. Default=";". E.g. 494;492
#' @param seq_number The annotation frame that contains the sequence after 
#' delimitation. E.g. The sequence "RSRYRTTSSANNPN" is contained in the 4th 
#' annotation frame of the following annotation: 
#' "A0A096MIX2|Ddx17|494|RSRYRTTSSANNPN" and would therefore set seq_number=4.
#' Default=4
#' @param replace Replace a letter that describes sequences outside of the 
#' protein after centering on the phosphosite (e.g X in XXXMERSTRELCLNF). Use in
#'  combination with replace_search and replace_with to replace amino acids. 
#'  Options are "TRUE" or "FALSE". Default="FALSE".
#' @param replace_search Amino Acid to search for when replacing sequences. 
#' Default="X"
#' @param replace_with Amino Acid to replace with when replacing sequences. 
#' Default="_"
#' @param verbose Print progress to screen. Default=FALSE
#'
#' @examples
#' ## Extract peptide sequences from annotation data:
#' data(example_phosphoproteome)
#'
#' ## A0A096MJ61|NA|89|PRRVRNLSAVLAART
#' ## The following will extract all the uniquely annotated peptide
#' ## sequences from the "annotation" column and place these in the
#' ## "peptide" column. Where multi-mapped peptide sequences are input,
#' ## these are placed on a new line.
#' ##
#' ## Here, sequences with a "X" and also replaced with a "_". This is ensure 
#' ## that PWMs are built correctly.
#'
#' ## Sample data for demonstration:
#' sample_data <- head(example_phosphoproteome)
#' annotated_data <- cleanAnnotation(input_data = sample_data,
#'                                    annotation_delimiter = "|",
#'                                    multi_protein_delimiter = ":",
#'                                    multi_site_delimiter = ";",
#'                                    seq_number = 4,
#'                                    replace = TRUE,
#'                                    replace_search = "X",
#'                                    replace_with = "_")
#'
#'## Return the annotated data with extracted peptides:
#' head(annotated_data)
#'
#' @return A data.table with the peptides extracted from the annotation column
#'
#' @export cleanAnnotation
#' @importFrom data.table data.table
#' @importFrom sqldf sqldf

cleanAnnotation <- function(input_data = NULL,
                            annotation_delimiter = "|",
                            multi_protein_delimiter = ":",
                            multi_site_delimiter = ";",
                            seq_number = 4,
                            replace = FALSE,
                            replace_search = "X",
                            replace_with = "_",
                            verbose = FALSE) {
  #----------------------------------------------
  #format checks:
  if (is.null(input_data))
    stop("input_data not provided; you must provide an input table")
  if (!is.data.frame(input_data))
    stop("input_data is not a data.frame; you must provide an
         input table")
  if (replace == TRUE) {
    if (!is.character(replace_search) && !is.character(replace_with)) {
      stop("replace_search AND replace_with MUST both be characters")
    }
  }
  #----------------------------------------------
  colnames(input_data) <- c("annotation", "peptide", "fc", "pval")
  #extract the annotation from the table:
  data_annotation <- as.character(input_data[, 1])
  #1. obtain sequences of interest
  seq_list <-
    strsplit(data_annotation, annotation_delimiter, fixed = TRUE)
  #get the nth element (seq_number) from the list (that contains the sequences)
  seqs <- sapply(seq_list, `[`, seq_number)
  #2. multi-site assignment
  seq_list <-
    strsplit(as.character(seqs), multi_site_delimiter, fixed = TRUE)
  seqs <- unlist(seq_list)
  #3. multi-gene assignment
  seq_list <-
    strsplit(as.character(seqs), multi_protein_delimiter, fixed = TRUE)
  seqs <- unlist(seq_list)
  #4. keep unique sequences for remapping
  seqs <- unique(seqs)
  #5. merge (i.e. remap) with input.data
  dt_in <- data.table(input_data)
  dt_seqs <- data.table(seqs)
  data_out <- data.table(
    sqldf(
      "select *
      from dt_in a inner join dt_seqs b
      on a.annotation like '%' || b.seqs || '%'",
      drv = "SQLite"
    )
  )
  data_out <-
    data.frame(
      "annotation" = data_out$annotation,
      "peptide" = data_out$seqs,
      "fc" = data_out$fc,
      "pval" = data_out$pval
    )
  
  #6. option for reassignment of letters, incase wild-card differs
  if (replace == TRUE) {
    if (verbose) {
      message("Replacing all",
              replace_search,
              "with",
              replace_with
      )
    }
    data_out$peptide <-
      gsub(replace_search, replace_with, data_out$peptide)
  }
  return(unique(data_out))
}

# Helper function for trimming centered sequences
#
# @description Helper function for trimming sequences
# @param seqs_to_trim Vector of sequences to trim
# @param seq_length Full length of substrate sequence (default is 15). Will be 
# trimmed automatically or report error if sequences in kinase_table are not 
# long enough.
# @param verbose Print progress to screen. Default = FALSE
#
# @return Trimmed sequences or HALT

trimSeqs <- function(seqs_to_trim = NULL,
                     seq_length = NULL,
                     verbose = FALSE) {
  #check minimum sequence length is compatible
  min_seq <- min(nchar(seqs_to_trim))
  
  if (min_seq < seq_length) {
    stop("You have selected to trim to", seq_length,
         ", but this is less than minimum sequence length, which is",
         min_seq, ". Trimming will not be performed. Reduce window size to",
         min_seq, "or lower to perform trimming"
    )
  }
  
  #if minimum sequence length is compatible - trim and/or leave same:
  if (min_seq >= seq_length) {
    if (verbose) {
      message("Trimming sequences to", seq_length, "total AA width")
    }
    
    seq_diff <- (min_seq - seq_length) / 2
    trimmed_seqs <-
      substr(seqs_to_trim, 1 + seq_diff, min_seq - seq_diff)
  }
  return(trimmed_seqs)
}

#' View motif
#'
#' @description View information content for each position of the PWM.
#' Information content is modelled using Shannon's Entropy Model. The maximum
#' information content is therefore log2(n), where n is the number of amino
#' acids. Colors of Amino Acids are in accordance with the Lesk scheme.
#' @param pwm_in View a PWM provided using the buildPWM. Default = NULL
#' @param which_pwm If pwms are input (outputs of buildPWM), a kinase name must
#' match a name in pwms$kinase$kinase list of names. Default = NULL
#' @param fontsize Font size to use on x and y axis. Default = 10
#' @param view_pwm View the PWM. Default = FALSE
#' @param color_scheme Which color scheme to use for Amino Acid Groups. Options
#' are "lesk" or "shapely". Default = "shapely"
#' @param convert_PWM pwm_in is a matrix of counts at position. TRUE will 
#' convert this matrix to a PWM. Default = FALSE
#' @param pseudo Small amount added to the PWM model, where zero's exist, to 
#' avoid log zero. Default = 0.01
#' @param correction_factor Number of sequences used to infer the PWM. 
#' This can be used where a small number of sequences were used to build the 
#' model and included as E_n in the Shannon's Entropy Model. Default = NULL
#'
#' @examples
#' ## Build PWM models from phosphositeplus data with default of minimum
#' ## of 10 substrate sequences for building a PWM model.
#' data(phosphositeplus_human)
#' ##randomly sample 1000 substrates for demonstration.
#' set.seed(1)
#' sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
#' 1000),]
#' pwms <- buildPWM(sample_pwm)
#'
#' ## Data frame of models built and number of sequences used to build each
#' ## PWM model:
#' head(pwms$kinase)
#' ## Will not visualise the motif
#' CAMK2A_motif <- viewPWM(pwm_in = pwms, 
#'                         which_pwm = "CAMK2A",
#'                         view_pwm = FALSE)
#' # Use view_pwm = TRUE to view the motif
#' @return Visualisation of a motif, scaled on bits and two tables. 1) pwm: 
#' corresponding to the PWM from pwm and 2) pwm_bits: corresponding to the
#' conversion to bits.
#'
#' @export viewPWM
#' 
#' @importFrom grid grid.newpage grid.polygon gpar pushViewport plotViewport
#' @importFrom grid dataViewport grid.xaxis gpar grid.yaxis
#' @importFrom grDevices rgb

#this function is for delivering a PWM to a frame:
viewPWM <- function(pwm_in = NULL, 
                    which_pwm = NULL,
                    fontsize = 10,
                    view_pwm = FALSE,
                    pseudo = 0.01,
                    convert_PWM = FALSE,
                    color_scheme = "shapely",
                    correction_factor = NULL){
  
  #----------------------------------------------
  if (is.null(pwm_in) && convert_PWM == FALSE)
    stop(
      "pwm_in not provided; you must provide an input table containing
      computed position weight matrices using buildPWM()"
    )
  if (!is.null(pwm_in) && is.null(which_pwm))
    stop(
      "which_pwm not provided; you must provide a PWM to search for"
    )
  if (!is.null(pwm_in) && !is.list(pwm_in))
    stop(
      "pwm_in is not a list format; something has gone wrong. Make sure
      you compute the position weight matrices using buildPWM()"
    )
  if (!(color_scheme %in% c("shapely", "lesk")))
    stop("color_scheme must be either shapely or lesk")
  
  #----------------------------------------------
  
  # PWM already converted
  if(convert_PWM == FALSE){
    # if a list is input and to be extracted:
    rownames(pwm_in$kinase) <- seq_len(nrow(pwm_in$kinase))
    which_pwm <- pwm_in$kinase[pwm_in$kinase$kinase==which_pwm,]
    which_pwm <- pwm_in$pwm[[as.numeric(rownames(which_pwm))]]
  }
  # data is input as count table
  if(convert_PWM == TRUE){
      which_pwm <- pwm_in
      #which_pwm <- SPRK2_motif_1
      wild_card <- "_"
      uniq_AA <- c(wild_card, "A", "C", "D", "E", "F", "G", "H", "I", "K", "L", 
                   "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
      missing_AA <- setdiff(uniq_AA, rownames(which_pwm))
      missing_AA <- data.frame(matrix(data = 0, ncol = ncol(which_pwm), 
                           nrow = length(missing_AA)), row.names = missing_AA)
      colnames(missing_AA) <- colnames(which_pwm)
      which_pwm <- rbind(missing_AA, which_pwm)
      # remove wildcard from motif scoring
      which_pwm <- which_pwm[!rownames(which_pwm) %in% c(wild_card),]
      which_pwm <- which_pwm[order(rownames(which_pwm)),]
      # col sums
      col_counts <- apply(which_pwm, 2, sum, na.rm = TRUE)
      # conver to PPM
      ppm <- sapply(1:ncol(which_pwm), function(i) 
        which_pwm[,i] / col_counts[i])
      rownames(ppm) <- rownames(which_pwm)
      # Generate Position Weight Matrix
      # addition of pseudo count
      which_pwm <- log2((ppm / (1 / nrow(ppm)))+pseudo)
      
      colnames(which_pwm) <- paste("p", seq_len(ncol(which_pwm)), sep = "")
    }
    
  # select window width = size of motif
  window_size <- ncol(which_pwm)
  half_window <- (window_size-1)/2
  # inverse log
  pwm <- 2^(which_pwm)/20
  # max bits, uncertainty with 20 AAs
  max_bits <- log2((1/0.05)+pseudo) 
  # Calculate Shannon Entropy, uncertainty at position (l)
  H_l <- pwm * log2(pwm)
  H_l <- -apply(H_l, 2, sum)
  if(is.null(correction_factor)){
    R_l <- max_bits - (H_l)# + e_n)
  }
  if(!is.null(correction_factor)){
    e_n <- correction_factor
    R_l <- max_bits - ((H_l) + e_n)
  }
  # multiplication step for vector, each col, to find Height (information)
  Height <- sapply(1:ncol(pwm), function(i) 
                    pwm[,i] * R_l[i])
  colnames(Height) <- colnames(pwm)
  # transform pwm to the Height
  pwm <- Height
  y_max <- 1
  # convert to 0-1 scale
  pwm <- pwm/max_bits
  
  if(view_pwm == TRUE){
    # start new grid for viewing and set up margins
    grid.newpage()
    pushViewport(plotViewport(margins = c(4,3,2,2)))
    pushViewport(dataViewport(xscale = c(0,1), yscale = c(0,1), name="vp1"))
    # label axis
    grid.xaxis(at = (seq(0,(window_size-1))/window_size)+((1/(window_size-1))/2),
               label = c(-half_window:half_window), 
               gp = gpar(fontsize = fontsize))
    grid.yaxis(at = c(0, 0.5, 1),
               label = c(0, round(max_bits/2, 2), round(max_bits, 2)), 
               gp = gpar(fontsize = fontsize))
    # add letters to the grid
    sapply(seq_len(ncol(pwm)), function(i)
      sapply(seq_len(nrow(pwm)), function(j)
        add.letter(letter = rownames(pwm[order(pwm[,i]),])[j], 
                   x.position = i, 
                   y.position = sum(pwm[order(pwm[,i]),][1:j-1,i]),
                   prob = pwm[order(pwm[,i]),][j,i],
                   window.size = window_size,
                   color_scheme = color_scheme
                   )
      )
    )
  }
return(list("pwm" = which_pwm, 
            "pwm_bits" = pwm))
}


# helper function to select the letters
add.letter <- function(letter, 
                       x.position, 
                       y.position, 
                       prob, 
                       window.size,
                       color_scheme){

  if(letter == "A"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 255,
                      green = 150,
                      blue = 0, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 200,
                      green = 200,
                      blue = 200, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,4,6,10,8,6.8,3.2,2,0,3.6,5,6.4,3.6)/10, 
                y = c(0,10,10,0,0,3,3,0,0,4,7.5,4,4)/10, 
                id = rep(1, 13), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
                )
    }
  if(letter == "C"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 230,
                      green = 230,
                      blue = 0, 
                      maxColorValue = 255)
    }
    ang <- seq((pi/2), (2*pi)+(pi/2), length = 360)
    x.out <- ((sin(ang))+1)/2; x.out <- x.out[40:(length(x.out)-40)]
    y.out <- ((cos(ang))+1)/2; y.out <- y.out[40:(length(y.out)-40)]
    x.in <- ((sin(ang)*0.7)+1)/2;x.in <- x.in[40:(length(x.in)-40)]
    y.in <- ((cos(ang)*0.7)+1)/2;y.in <- y.in[40:(length(y.in)-40)]
    add.polygon(x = c(x.out, rev(x.in)), 
                y = c(y.out, rev(y.in)), 
                id = rep(1,length(c(x.out, rev(x.in)))), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
                )
    }
  if(letter == "D"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 230,
                      green = 10,
                      blue = 10, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 230,
                      green = 10,
                      blue = 10, 
                      maxColorValue = 255)
    }
    ang <- seq((pi), (2*pi)+(pi), length = 360)
    x.out <- (((sin(ang)))/2)+0.5;x.out <- x.out[180:(length(x.out))]
    y.out <- ((cos(ang))+1)/2;y.out <- y.out[180:(length(y.out))]
    x.out <- c(x.out, 0, 0);y.out <- c(y.out, 0, 1)
    x.in <- (((((sin(ang)))/2)+0.5)*0.7)+0.15;x.in <- x.in[180:(length(x.in))]
    y.in <- ((((cos(ang))*0.7)+1)/2);y.in <- y.in[180:(length(y.in))]
    x.in <- c(x.in, 1-max(x.in), 1-max(x.in));y.in <- c(y.in, min(y.in), max(y.in))
    add.polygon(x = c(x.out, x.in), 
                y = c(y.out, y.in), 
                id = c(rep(1,length(x.out)), 
                       rep(2,length(x.in))), 
                fill = c(fill_rgb, 
                         rgb(red = 255,
                             green = 255,
                             blue = 255, 
                             maxColorValue = 255)),
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "E"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 230,
                      green = 10,
                      blue = 10, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 230,
                      green = 10,
                      blue = 10, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,10,10,2,2,6,6,2,2,10,10)/10, 
                y = c(0,10,10,8,8,6,6,4,4,2,2,0)/10, 
                id = rep(1, 12), 
                fill = fill_rgb, 
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "F"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 50,
                      green = 50,
                      blue = 170, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,10,10,2,2,10,10,2,2)/10, 
                y = c(0,10,10,8,8,6,6,4,4,0)/10, 
                id = rep(1, 10), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "G"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 255,
                      green = 150,
                      blue = 0, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 235,
                      green = 235,
                      blue = 235, 
                      maxColorValue = 255)
    }
    ang <- seq((pi/2), (2*pi)+(pi/2), length = 360)
    x.out <- ((sin(ang))+1)/2;x.out <- x.out[40:(length(x.out)-40)]
    y.out <- ((cos(ang))+1)/2;y.out <- y.out[40:(length(y.out)-40)]
    x.in <- ((sin(ang)*0.7)+1)/2;x.in <- x.in[40:(length(x.in)-40)]
    y.in <- ((cos(ang)*0.7)+1)/2;y.in <- y.in[40:(length(y.in)-40)]
    x <- c(x.in[1], x.in[1], 0.5,0.5, max(x.out),  x.out, rev(x.in))
    y <- c( y.in[1],0.4,0.4,0.6,0.6, y.out, rev(y.in))
    add.polygon(x = x, 
                y = y, 
                id = rep(1,length(x)), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "H"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 255,
                      green = 255,
                      blue = 255, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 130,
                      green = 130,
                      blue = 210, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,2,2,8,8,10,10,8,8,2,2,0)/10, 
                y = c(0,10,10,6,6,10,10,0,0,4,4,0,0)/10, 
                id = rep(1, 13), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "I"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,4,4,0,0,10,10,6,6,10,10,0)/10, 
                y = c(0,2,2,8,8,10,10,8,8,2,2,0,0)/10, 
                id = rep(1, 13), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "K"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 20,
                      green = 90,
                      blue = 255, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 20,
                      green = 90,
                      blue = 255, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,2,2,7,10,4,10,7,2,2,0)/10, 
                y = c(0,10,10,6,10,10,5,0,0,4,0,0)/10, 
                id = rep(1, 12), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "L"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,2,2,10,10,0)/10, 
                y = c(0,10,10,2,2,0,0)/10, 
                id = rep(1, 7), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "M"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 230,
                      green = 230,
                      blue = 0, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,2,5,8,10,10,8,8,5,2,2,0)/10, 
                y = c(0,10,10,6,10,10,0,0,5,2,5,0,0)/10, 
                id = rep(1, 13), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "N"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 255,
                      green = 0,
                      blue = 255, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 0,
                      green = 220,
                      blue = 220, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,0,3,8,8,10,10,7,2,2,0)/10, 
                y = c(0,10,10,3,10,10,0,0,7,0,0)/10, 
                id = rep(1, 11), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "P"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 250,
                      green = 150,
                      blue = 130, 
                      maxColorValue = 255)
    }
    ang <- seq((pi),(2*pi)+(pi), length = 360)
    x.out <- (((sin(ang)))/2)+0.5
    x.out <- c(x.out[180:(length(x.out))],0.2,0.2,0,0) 
    y.out <- (((cos(ang))+1)/4)+0.5
    y.out <- c(y.out[180:(length(y.out))],0.5,0,0,1)
    x.in <- (((((sin(ang)))/2)+0.5)*0.5)+0.30
    x.in <- c(x.in[180:(length(x.in))],0.2,0.2)
    y.in <- (((((cos(ang))+1)/4)+0.5)*0.5)+0.375
    y.in <- c(y.in[180:(length(y.in))],min(y.in), max(y.in))
    add.polygon(x = c(x.out, x.in), 
                y = c(y.out, y.in), 
                id = c(rep(1,length(x.out)), 
                       rep(2,length(x.in))), 
                fill = c(fill_rgb,
                         rgb(red = 255,
                             green = 255,
                             blue = 255, 
                             maxColorValue = 255)),
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "Q"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 255,
                      green = 0,
                      blue = 255, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 0,
                      green = 220,
                      blue = 220, 
                      maxColorValue = 255)
    }
    ang <- seq((pi/2), (2*pi)+(pi/2), length = 360)
    x.out <- ((sin(ang))+1)/2
    y.out <- ((cos(ang))+1)/2
    x.in <- ((sin(ang)*0.7)+1)/2
    y.in <- ((cos(ang)*0.7)+1)/2
    x <- c(x.out, rev(x.in))*0.9
    y <- c(y.out, rev(y.in))
    x.q <- c(0.9, 0.55, 0.65, 1)
    y.q <- c(0, 0.35, 0.45, 0.1)
    add.polygon(x = c(x, x.q), 
                y = c(y, y.q), 
                id = c(rep(1,length(x)), 
                       rep(2,length(x.q))), 
                fill = c(fill_rgb,
                         fill_rgb),
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "R"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 20,
                      green = 90,
                      blue = 255, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 20,
                      green = 90,
                      blue = 255, 
                      maxColorValue = 255)
    }
    ang <- seq((pi), (2*pi)+(pi), length = 360)
    x.out <- (((sin(ang)))/2)+0.5
    x.out <- c(x.out[180:(length(x.out))], 0.2, 0.2, 0, 0)
    y.out <- (((cos(ang))+1)/4)+0.5
    y.out <- c(y.out[180:(length(y.out))], 0.5, 0, 0, 1)
    x.in <- (((((sin(ang)))/2)+0.5)*0.5)+0.30
    x.in <- c(x.in[180:(length(x.in))],0.2, 0.2)
    y.in <- (((((cos(ang))+1)/4)+0.5)*0.5)+0.375
    y.in <- y.in[180:(length(y.in))];
    y.in <- c(y.in, min(y.in), max(y.in))
    x.r <- c(0.8, 0.4, 0.6, 1);y.r <- c(0, 0.55, 0.55, 0)
    add.polygon(x = c(x.out, x.in, x.r), 
                y = c(y.out, y.in, y.r), 
                id = c(rep(1,length(x.out)), 
                       rep(2,length(x.in)), 
                       rep(3,length(x.r))), 
                fill = c(fill_rgb,
                         rgb(red = 255,
                             green = 255,
                             blue = 255, 
                             maxColorValue = 255),
                         fill_rgb),
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "S"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 250,
                      green = 150,
                      blue = 0, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 250,
                      green = 150,
                      blue = 0, 
                      maxColorValue = 255)
    }
    angle1 <- seq((pi), (2*pi)+(pi), length = 360)
    #outer and inner
    x.out <- (((sin(angle1)))/2)+0.5
    x.out <- (x.out[100:(length(x.out))]*-1)+1
    y.out <- (((cos(angle1))+1)*0.3)
    y.out <- y.out+(1-max(y.out))
    y.out <- y.out[100:(length(y.out))]
    x.out.b <- (x.out*-1)+1
    y.out.b <- (y.out*-1)+1
    x.in <- (x.out*0.5)+0.25
    y.in <- (y.out*0.4)
    y.in <- y.in + (max(y.out.b)-min(y.in))
    x.in.b <- (x.in*-1)+1
    y.in.b <- (y.in*-1)+1
    x <- c(x.out, rev(x.in.b), x.out.b, rev(x.in))
    y <- c(y.out, rev(y.in.b), y.out.b, rev(y.in))
    add.polygon(x = x, 
                y = y, 
                id = rep(1,length(x)), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "T"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 250,
                      green = 150,
                      blue = 0,
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 250,
                      green = 150,
                      blue = 0, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0,10,10,6,6,4,4,0)/10, 
                y = c(10,10,8,8,0,0,8,8)/10, 
                id = rep(1, 8), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "V"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15,
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(0.4,0,0.2,0.5,0.8,1,0.6), 
                y = c(0,1,1,0.2,1,1,0), 
                id = rep(1, 7), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "W"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15, 
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 180,
                      green = 90,
                      blue = 180,
                      maxColorValue = 255)
    }
    add.polygon(x = c(2,0,2,3.5,5,6.5,8,10,8,6,5,4)/10, 
                y = c(0,10,10,3,6,3,10,10,0,0,3,0)/10, 
                id = rep(1, 12), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "Y"){
    if(color_scheme == "lesk"){
      fill_rgb <- rgb(red = 15,
                      green = 130,
                      blue = 15,
                      maxColorValue = 255)
    }
    if(color_scheme == "shapely"){
      fill_rgb <- rgb(red = 50,
                      green = 50,
                      blue = 170, 
                      maxColorValue = 255)
    }
    add.polygon(x = c(4,4,0,2,5,8,10,6,6)/10, 
                y = c(0,5,10,10,6.5,10,10,5,0)/10, 
                id = rep(1, 9), 
                fill = fill_rgb,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "_"){
    add.polygon(x = , 
                y = , 
                id = , 
                fill = ,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
  if(letter == "X"){
    add.polygon(x = c(0,0,10,10)/10, 
                y = c(0,10,10,0)/10, 
                id = rep(1, 4), 
                fill = 1,
                x.factor = 1/window.size, 
                x.pos = x.position, 
                y.pos = y.position, 
                y.scale = prob
    )
  }
}

add.polygon <- function(x, y, id, fill, x.factor, x.pos, y.pos, y.scale){
  x.pos = x.pos-1
  grid.polygon(x=(x*x.factor)+(x.pos*x.factor),
               y=(y*y.scale)+y.pos,
               id = id,
               gp = gpar(fill = fill, col="transparent"))
}

#' Example phosphoproteome.
#'
#' A dataset containing annotated subtrate sequences derived
#' from XXX. See original publication for more
#' details: Engholm-Keller & Waardenberg AJ et al.
#'
#' @format A data frame with 6215 rows and 4 variables:
#' \describe{
#'   \item{annotation}{Annotation of phosphorylated peptides}
#'   \item{peptide}{blank - peptides need to be extracted from
#'   annotation}
#'   \item{fc}{Fold Change (log2)}
#'   \item{pval}{P-value for fold-change.}
#' }
"example_phosphoproteome"

#' KinSwingR: A package for predicting kinase activity
#'
#' This package provides functionality for kinase-subtrate prediction, and 
#' integration with phosphopeptide fold change and signficance to assess the 
#' local connectivity (swing) of kinase-substrate networks. The final output of 
#' KinSwingR is a score that is normalised and weighted for prediction of kinase
#'  activity.
#'
#' Contact a.waardenberg@gmail.com for questions relating to functionality.
#'
#' @section buildPWM function:
#' Builds PWMs for kinases from a table of kinases and known substrate 
#' sequences.
#'
#' @section scoreSequences function:
#' Score kinase PWMs matches against a set of peptide seqeuences.
#'
#' @section swing function:
#' Integrates kinase PWMs matches against peptide seqeuences and directionality 
#' as well as significance of peptides for prediction of kinase activity.
#'
#' @section cleanAnnotation function:
#' Function for extracting peptides from multimapped data
#'
#' @docType package
#' @name KinSwingR
#'
NULL

## ----echo=TRUE----------------------------------------------------------------
library(KinSwingR)
data(example_phosphoproteome)
data(phosphositeplus_human)

# View the datasets:
head(example_phosphoproteome)

head(phosphositeplus_human)

## sample 100 data points for demonstration
sample_data <- head(example_phosphoproteome, 1000)

# Random sample for demosntration purposes
set.seed(1234)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 1000),]

# for visualising a motif, sample only CAMK2A
CAMK2A_example <- phosphositeplus_human[phosphositeplus_human[,1]== "CAMK2A",] 


## ----echo=TRUE----------------------------------------------------------------
annotated_data <- cleanAnnotation(input_data = sample_data, 
                                  annotation_delimiter = "|",
                                  multi_protein_delimiter = ":", 
                                  multi_site_delimiter = ";",
                                  seq_number = 4, 
                                  replace = TRUE, 
                                  replace_search = "X",
                                  replace_with = "_")

head(annotated_data)


## ----echo=TRUE----------------------------------------------------------------
pwms <- buildPWM(sample_pwm)

## ----echo=TRUE----------------------------------------------------------------
head(pwms$kinase)

## ----echo=TRUE----------------------------------------------------------------
CAMK2A_pwm <- buildPWM(CAMK2A_example)
CAMK2A <- viewPWM(CAMK2A_pwm, 
                  which_pwm="CAMK2A",
                  view_pwm = TRUE,
                  color_scheme = "shapely")

## ----echo=TRUE----------------------------------------------------------------
# As an example of control over multi-core processing
# load BiocParallel library
library(BiocParallel)
# finally set/register the number of cores to use
register(SnowParam(workers = 4))

# set seed for reproducible results
set.seed(1234)
scores <- scoreSequences(input_data = annotated_data, 
                         pwm_in = pwms,
                         n = 100)

## ----echo=TRUE----------------------------------------------------------------
# after loading BiocParallel library, set/register the number of cores to use
register(SnowParam(workers = 4))

# set seed for reproducible results
set.seed(1234)
swing_out <- swing(input_data = annotated_data, 
                   pwm_in = pwms, 
                   pwm_scores = scores,
                   permutations = 10)

# This will produce two tables, one is a network for use with e.g. Cytoscape and the other is the scores. To access the scores:
head(swing_out$scores)

#' Score substrate sequences for matches to kinase Position Weight Matrices 
#' (PWMs)
#'
#' @description Scores each input sequence for a match against all PWMs provided
#'  from buildPWM() and generates p-values for scores. The output of this 
#'  function is to be used for building the swing metric, the predicted activity
#'   of kinases.
#' @param input_data A data.frame of phoshopeptide data. Must contain 4 columns
#'  and the following format must be adhered to. Column 1 - Annotation, Column 2
#'   - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
#'   - p-value [0-1]
#' @param pwm_in List of PWMs created using buildPWM()
#' @param background Option to provide a data.frame of peptides to use as 
#' background. If providing a background as a table, this must contain two 
#' columns; Column 1 - Annotation, Column 2 - centered peptide sequence. These 
#' must be centered. OR generate a random background for PWM scoring from the
#' input list - background = random. Default: "random"
#' @param n Number of permutations to perform for generating background. 
#' Default: "1000"
#' @param force_trim This function will detect if a peptide sequence is of 
#' different length to the PWM models generated (provided in pwm_in) and trim 
#' the input sequences to the same length as the PWM models. If a background is 
#' provided, this will also be trimmed to the same width as the PWM models. 
#' Options are: "TRUE, FALSE". Default = FALSE
#' @param verbose Turn verbosity on/off. To turn on, verbose=TRUE. Options are: 
#' "TRUE, FALSE". Default = FALSE
#'
#' @examples
#' ## import data
#' data(example_phosphoproteome)
#' data(phosphositeplus_human)
#'
#' ## clean up the annotations
#' ## sample 100 data points for demonstration
#' sample_data <- head(example_phosphoproteome, 100)
#' annotated_data <- cleanAnnotation(input_data = sample_data)
#'
#' ## build the PWM models:
#' set.seed(1234)
#' sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
#' 1000),]
#' pwms <- buildPWM(sample_pwm)
#'
#' ## score the PWM - substrate matches
#' ## Using a "random" background, to calculate the p-value of the matches
#' ## Using n=10 for demonstration
#' ## set.seed for reproducibility
#' set.seed(1234)
#' substrate_scores <- scoreSequences(input_data = annotated_data,
#'                                    pwm_in = pwms,
#'                                    background = "random",
#'                                    n = 10)
#'
#' @return A list with 3 elements: 1) PWM-substrate scores: 
#' substrate_scores$peptide_scores, 2) PWM-substrate p-values: 
#' substrate_scores$peptide_p 3) Background used for reproducibility: 
#' substrate_scores$background 4) input_data is returned in the case that it was
#' trimmed.
#'
#' @export scoreSequences
#' @importFrom BiocParallel bplapply

scoreSequences <- function(input_data = NULL,
                           pwm_in = NULL,
                           background = "random",
                           n = 1000,
                           force_trim = FALSE,
                           verbose = FALSE) {
  if (verbose) {
    message("Scoring,", nrow(input_data),
            "sequences provided, against", length(pwm_in[[1]]),
            "PWM models")
    message("Parameters selected:\n", 
            "n =", n, "\n",
            "force_trim =", force_trim, "\n",
            "verbose =", verbose, "\n"
    )
  }
  
  #----------------------------------------------
  #format checks:
  if (is.null(input_data))
    stop("input_data not provided; you must provide an input table")
  if (!is.data.frame(input_data))
    stop("input_data is not a data.frame; you must provide an
         input table")
  if (is.null(pwm_in))
    stop(
      "pwm_in not provided; you must provide an input table containing
      computed position weight matrices using buildPWM()"
    )
  if (!is.list(pwm_in))
    stop(
      "pwm_in is not a list format; something has gone wrong. Make sure
      you compute the position weight matrices using buildPWM()"
    )
  #min peptide sequence length of input data
  min_peptide_seq <- min(nchar(as.character(input_data[, 2])))
  #length of PWM models
  min_pwm_length <- min(sapply(seq_len(length(pwm_in[[1]])), function(X)
    ncol(data.frame(pwm_in[[1]][X]))))
  if ((min_peptide_seq > min_pwm_length) & force_trim == FALSE)
    stop(
      "Centered peptide sequence is greater than length of position weight 
      matrix. Check data and/or consider using force_trim=TRUE."
    )
  if ((min_peptide_seq < min_pwm_length) & force_trim == FALSE)
    stop(
      paste(
        "Centered peptide sequence is less than the length of PWMs.
        Rebuild PWM with length of,",
        min_peptide_seq,
        ",and rerun.",
        sep = ""
      )
    )
  if ((min_peptide_seq > min_pwm_length) & force_trim == TRUE) {
    if (verbose) {
      message("trimming input_data sequences to minimum PWM length, which is,",
              min_pwm_length
      )
    }
    #trim  seqs to the min. sequence length in PWMs
    input_data[, 2] <-
      trimSeqs(
        seqs_to_trim = as.character(input_data[, 2]),
        seq_length = min_pwm_length,
        verbose = verbose
      )
  }
  if (background != "random" && !is.data.frame(background)) {
    stop(
      "Background is not in data.frame() format. Use backgound=random or
      provide a table of peptides as background in data.frame format with
      two columns, Column1 = Annotation, Column2 = Centered peptide"
    )
  }
  if (background != "random" && is.data.frame(background)) {
    #remove NA's
    background <- background[!is.na(background[, 2]),]
    #check peptide sequence length of background data if provided:
    min_peptide_seq_bg <- min(nchar(as.character(background[, 2])))
    if ((min_peptide_seq_bg > min_pwm_length) & force_trim == FALSE)
      stop(
        "Centered peptide sequence of BACKGROUND is greater than width of
        position weight matrix. Set option to force trim to continue."
      )
    if ((min_peptide_seq_bg < min_pwm_length) & force_trim == FALSE)
      stop(
        "Centered peptide sequence if BACKGROUND is less than length of
        position weight matrix. Rebuild PWM with length of X and rerun."
      )
    if (n > nrow(background))
      stop("Background provided is smaller than n selected. Reduce n")
    
    if ((min_peptide_seq_bg > min_pwm_length) & force_trim == FALSE) {
      if (verbose) {
        message("trimming BACKGROUND sequences provided to minimum PWM length,
                which is,",
                min_pwm_length
        )
      }
      #trim seqs to the min. sequence length inPWMs
      background[, 2] <-
        trimSeqs(
          seqs_to_trim = as.character(background[, 2]),
          seq_length = min_pwm_length,
          verbose = verbose
        )
    }
    
    }
  #check "n" not smaller than input dataset
  if (background == "random" && n > nrow(input_data)) {
    stop("n is larger than dataset. Reduce n random sampling")
  }
  
  #----------------------------------------------
  #1. for each peptide, score against all kinase PWMs provided:
  if (verbose) {
    message("[Step1/3] : Scoring peptide sequences against PWMs")
  }
  
  peptide_scores <-
    bplapply(seq_along(pwm_in[[2]]$kinase), function(i)
      sapply(seq_len(nrow(input_data)), function(j)
        seqScore(as.character(input_data[j, 2]), pwm_in[[1]][[i]])))
  
  names(peptide_scores) <- pwm_in[[2]]$kinase
  peptide_scores <-
    cbind("annotation" = input_data[, 1],
          "peptide" = input_data[, 2],
          data.frame(peptide_scores))
  
  #2. extract background set of size "n"
  # if no background is provided:
  if (background == "random" && !is.data.frame(background)) {
    if (verbose) {
      message("[Step2/3] : Random background generated from ", n, " peptides")
    }
    rand_bg <- sample(rownames(peptide_scores), n)
    background_scores <-
      peptide_scores[rownames(peptide_scores) %in% rand_bg, ]
  }
  # or if background is provided
  if (background != "random" && is.data.frame(background)) {
    if (verbose) {
      message("[Step2/3] : Background provided; calculating PWM scores against 
              background generated from ", n, " peptides"
      )
    }
    rand_bg <- sample(rownames(background), n)
    rand_bg <- background[rownames(background) %in% rand_bg, ]
    
    background_scores <-
      bplapply(seq_along(pwm_in[[2]]$kinase), function(i)
        sapply(seq_len(nrow(rand_bg)), function(j)
          seqScore(as.character(rand_bg[j, 2]), pwm_in[[1]][[i]])))
    
    names(background_scores) <- pwm_in[[2]]$kinase
    background_scores <- cbind(
      "annotation" = rand_bg[, 1],
      "peptide" = rand_bg[, 2],
      data.frame(background_scores)
    )
    
    }
  
  #3. compute p-values:
  if (verbose) {
    message("[Step3/3] : Computing p-values for peptide scores:PWM scores")
  }
  
  # multi-core
  p_table <- bplapply(3:ncol(peptide_scores), function(i)
    sapply(seq_len(length(peptide_scores[, i])), function(j)
      (sum(
        ifelse(
          as.numeric(background_scores[, i]) >
            as.numeric(peptide_scores[, i][j]),
          1, 0 )) + 1) / (length(as.numeric(background_scores[, i])) + 1)))
  
  names(p_table) <- colnames(3:ncol(peptide_scores))
  p_table <- data.frame(peptide_scores[1:2], data.frame(p_table))
  colnames(p_table) <- gsub("score_", "p_", colnames(peptide_scores))
  rownames(p_table) <- rownames(peptide_scores)
  
  if (verbose) {
    message("[Finished]")
  }
  return(
    list(
      "peptide_scores" = peptide_scores,
      "peptide_p" = p_table,
      "background" = background_scores,
      "input_data" = input_data
    )
  )
  }

# helper function to calculate PWM match scores (vectorised)
seqScore <- function(input_seq, pwm) {
  # vector of letters of sequence to score
  input_seq <- unlist(strsplit(input_seq, ""))
  idx <- input_seq != "_"
  ridx <- match(input_seq[idx], rownames(pwm))
  cidx <- seq_along(input_seq)[idx]
  # return sum of scores for each matched AA
  # create a row/column index (which row matches each AA in string)
  # extracts the corresponding scores and then sums for the total score
  sum(pwm[cbind(ridx, cidx)])
}

#' Generate Position Weight Matrices (PWMs)
#'
#' @description Generate Position Weight Matrices (PWMs) for a table containing 
#' centered substrate peptide sequences for a list of kinases. The output of 
#' this function is to be used for scoring PWM matches to peptides via 
#' scoreSequences()
#' @param kinase_table A data.frame of substrate sequences and kinase names. 
#' Format of data must be as follows: column 1 - kinase/kinase family 
#' name/GeneID, column 2 - centered peptide seqeuence.
#' @param wild_card Letter to describe sequences that are outside of the protein
#'  after centering on the phosphosite (e.g ___MERSTRELCLNF). Default: "_".
#' @param substrate_length Full length of substrate sequence (default is 15). 
#' Will be trimmed automatically or report error if sequences in kinase_table 
#' are not long enough.
#' @param substrates_n Number of sequences used to build a PWM model. Low 
#' sequence counts will produce poor representative PWM models. Default: "10"
#' @param pseudo Small number to add to values for PWM log transformation to 
#' prevent log transformation of zero. Default = 0.01
#' @param remove_center Remove all peptide seqeuences with the central amino 
#' acid matching a character (e.g. "y"). Default = FALSE
#' @param verbose Print progress to screen. Default=FALSE
#'
#' @examples
#' ## Build PWM models from phosphositeplus data with default of minimum
#' ## of 10 substrate sequences for building a PWM model.
#'
#' data(phosphositeplus_human)
#'
#' ##randomly sample 1000 substrates for demonstration.
#' set.seed(1)
#' sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
#' 1000),]
#' pwms <- buildPWM(sample_pwm)
#'
#' ## Data frame of models built and number of sequences used to build each
#' ## PWM model:
#' head(pwms$kinase)
#'
#' @return Output is a list containing two tables, "pwm" and "kinase". To access
#'  PWMs: pwms$pwm and Table of Kinase and sequence counts: 
#'  pwms$kinase
#'
#' @export buildPWM

buildPWM <- function(kinase_table = NULL,
                     wild_card = "_",
                     substrate_length = 15,
                     substrates_n = 10,
                     pseudo = 0.01,
                     remove_center = FALSE,
                     verbose = FALSE) {
  #----------------------------------------------
  #format checks:
  if (is.null(kinase_table))
    stop("kinase_table not provided; you must provide an input table")
  if (!is.matrix(kinase_table))
    stop("kinase_table is not a table; you must provide an
         input table")
  #remove NA's
  kinase_table <- kinase_table[!is.na(kinase_table[, 2]),]
  #remove problematic characters (creates row/column name issues)
  kinase_table[,1] <- gsub("/", "_", kinase_table[,1])
  
  if (substrate_length < 3)
    stop(
      "substrate_length needs to be greater than 2; increase the
      length of substrate window size"
    )
  if ((lapply(substrate_length, "%%", 2) == 0) == TRUE)
    stop("substrate_length must be an odd number! I.e. centered sequence
         of ++X++")
  #----------------------------------------------
  #call trim_seq function and trim sequences:
  kinase_table[, 2] <-
    trimSeqs(kinase_table[, 2], 
             seq_length = substrate_length, 
             verbose = verbose)
  kinase_table[, 2] <- toupper(kinase_table[, 2])
  
  #remove peptides with a certain center character:
  if (remove_center != FALSE) {
    half_window <- (substrate_length + 1) / 2
    center_aa <-
      substr(kinase_table[, 2], half_window, half_window)
    if (verbose) {
      message("You have selected to remove",
              length(which(center_aa == toupper(remove_center))),
              "peptide sequences for building PWMs that contain a centered 
              letter of:",
              toupper(remove_center)
      )
    }
    kinase_table <-
      kinase_table[center_aa != toupper(remove_center),]
  }
  
  kinase_table <- unique(kinase_table)
  #----------------------------------------------
  
  if (verbose) {
    message(nrow(kinase_table), "unique kinase:substrate sequences in 
            table provided."
    )
  }
  
  #initialise table for kinase count data
  kinase_summary <-
    data.frame("kinase" = unique(as.character(kinase_table[, 1])), "n" = NA)
  
  kinase_summary$n <-
    c(sapply(seq_len(nrow(kinase_summary)), function(i)
      length(kinase_table[, 1][kinase_table[, 1] == kinase_summary[i, 1]])))
  
  #filter the list here which is used to build pwms:
  kinase_summary <-
    kinase_summary[kinase_summary$n >= substrates_n, ]
  
  #generate PWM list:
  pwm_list <-
    c(lapply(seq_len(nrow(kinase_summary)), function(i)
      scorePWM(
        as.character(kinase_table[, 2][kinase_table[, 1] == 
                                         kinase_summary[i, 1]]),
        substrate_length = substrate_length,
        wild_card = wild_card,
        pseudo = pseudo
      )))
  
  return(list("pwm" = pwm_list, "kinase" = kinase_summary))
  }

# this helper function performs the calculations for building the PWMs

scorePWM <- function(input_data,
                     substrate_length,
                     wild_card = "_",
                     pseudo = 0.01) {
  #reformat input_data; build a matrix of the sequences (split into individual
  # AA's) = substrate_length
  input_data <-
    data.frame(matrix(unlist(strsplit(input_data , "")) , 
                      ncol = substrate_length , byrow = TRUE))
  uniq_AA <- c(wild_card, "A", "C", "D", "E", "F", "G", "H", "I", "K", "L", 
               "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")
  # 1. Build PFM
  pwm <-
    t(cbind(sapply(seq_along(uniq_AA), function(i)
      (
        sapply(seq_len(ncol(input_data)), function(j)
          sum(
            ifelse(as.character(input_data[, j]) == 
                     as.character(uniq_AA[i]), 1, 0)
          ))
      ))))
  rownames(pwm) <- uniq_AA
  colnames(pwm) <- paste("p", seq_len(ncol(pwm)), sep = "")
  # remove wildcard from motif scoring
  pwm <- pwm[!rownames(pwm) %in% c(wild_card),]
  # 2. PPM calculation
  pwm_rownames <- rownames(pwm)
  pwm_colnames <- colnames(pwm)
  col_counts <- apply(pwm, 2, sum, na.rm = TRUE)
  pwm <- sapply(seq_len(ncol(pwm)), function(i) 
    pwm[,i] / col_counts[i])
  # carry labels forward
  rownames(pwm) <- pwm_rownames
  colnames(pwm) <- pwm_colnames
  # 3. Calculate Position Weight Matrix
  # addition of pseduo count to avoid log zero
  pwm <- log2((pwm / (1 / nrow(pwm)))+pseudo)
return(pwm)
}

#' Human kinase-substrates derived from PhosphositePlus.
#'
#' A dataset containing human kinases and subtrate sequences. See
#' original publication for more details: Hornbeck et al.
#' Nucleic Acids Res. 40:D261-70, 2012
#'
#' @format A data frame with 11985 rows and 2 variables:
#' \describe{
#'   \item{kinase}{human kinase gene symbol}
#'   \item{substrate}{centered substrate sequence for kinase}
#' }
#' @source \url{https://www.phosphosite.org/}
"phosphositeplus_human"

---
title: 'KinSwingR: Predicting kinase activity from phosphoproteomics data'
author: "Ashley J. Waardenberg"
date: 'Last modified: 2019-04-25. Compiled: `r Sys.Date()`'
output:
  html_document:
    highlight: tango
    number_sections: yes
    toc: yes
    toc_depth: 3
    toc_float:
      collapsed: no
      smooth_scroll: no
  pdf_document:
    toc: yes
    toc_depth: '3'
  word_document:
    toc: yes
    toc_depth: '3'
references:
- DOI: 10.1371/journal.pbio.3000170
  URL: 'https://doi.org/10.1371/journal.pbio.3000170'
  author:
  - family: Engholm-Keller
    given: K*
  - family: Waardenberg
    given: AJ*
  - family: Müller 
    given: JA
  - family: Wark 
    given: JR
  - family: Fernando  
    given: RN
  - family: Arthur  
    given: JA
  - family: Robinson 
    given: PJ
  - family: Dietrich  
    given: D
  - family: Schoch  
    given: S
  - family: Graham  
    given: ME
  id: Kasper2019
  issued:
    month: 3
    year: 2019
  publisher: PloS Biology
  title: The temporal profile of activity-dependent presynaptic phospho-signalling reveals long lasting patterns of post-stimulus regulation
  type: article-journal
vignette: >
  %\VignetteIndexEntry{KinSwingR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction to KinSwing

KinSwingR aims to predict kinase activity from phoshoproteomics data. It implements the alogorithm described in: @Kasper2019 (in greater detail below). KinSwingR predicts kinase activity by integrating kinase-substrate predictions and the fold change and signficance of change for peptide sequences obtained from phospho-proteomics studies. The score is based on the network connectivity of kinase-substrate networks and is weighted for the number of substrates as well as the size of the local network. P-values are provided to assess the significance of the KinSwing scores, which are determined through random permuations of the overall kinase-substrate network.

KinSwingR is implemented as 3 core functions:

+ **```buildPWM()```** builds position weight matrices (PWMs) from known kinase-substrate sequences
+ **```scoreSequences()```** score PWMs build using ```buildPWM()``` against input phosphoproteome data
+ **```swing()```** integrates PWM scores, direction of phosphopeptide change and significance of phosphopeptide change into a "swing" score.

The KinSwing score is a metric of kinase activity, ranging from positive to negative, and p-values are provided to determine significance.

Additional functions are also provided:

+ **```cleanAnnotation()```** function to tidy annotations and extract peptide sequences.
+ **```viewPWM()```** function to view PWM models


Detailed information for each of these functions can be accessed using the ```?``` command before the function of interest. E.g. ```?buildPWM```

# KinSwingR example workflow

We will now consider an example dataset to predict kinase activity. Kinase-substrate sequences and phosphoproteomics data are provided as example data in the KinSwingR package.

Begin by loading the KinSwingR library and the two data libraries included in the package.
```{r echo=TRUE}
library(KinSwingR)
data(example_phosphoproteome)
data(phosphositeplus_human)

# View the datasets:
head(example_phosphoproteome)

head(phosphositeplus_human)

## sample 100 data points for demonstration
sample_data <- head(example_phosphoproteome, 1000)

# Random sample for demosntration purposes
set.seed(1234)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 1000),]

# for visualising a motif, sample only CAMK2A
CAMK2A_example <- phosphositeplus_human[phosphositeplus_human[,1]== "CAMK2A",] 

```

## Extracting peptides for analysis
Where the centered peptide sequences (on the phosphosite of interest) are not provided in the format required for ```scoreSequences()``` (see the argument "input_data", in ?scoreSequences), these can be required to be extracted from another column of annotated data. NB. "input_data" table format must contain columns for "annotation", "peptide", "fold-change" and "p-values".

In the example dataset provided, ```example_phosphoproteome```, peptides have not been extracted into a stand-a-lone peptide column. ```cleanAnnotation()``` is provided as a function to extract peptides from annotation columns and place into the peptide column. 

In the example dataset, ```example_phosphoproteome```, the peptide sequence is the 4th component of the annotation, which corresponds to using the argument ```seq_number = 4``` below, and is seperated by ```|```, which corresponds to the argument ```annotation_delimiter = "|"```. In this case, the annotated data also contains multi-mapped and multi-site information. For example the following annotation ```A1L1I3|Numbl|263;270|PAQPGHVSPTPATTS;SPTPATTSPGEKGEA``` contains two peptides ```PAQPGHVSPTPATTS``` and ```SPTPATTSPGEKGEA``` that map to different sites from the same reference gene ```Numbl```, where the peptides are seperated by ```;```. The annotated data also includes multi-protein mapped (where a peptide could map to more than one protein - not shown) and contains ```X``` instead of ```_``` to indicate sequences that were outside of the length of the coding sequences. KinSwingR requires that these sequences outside of the coding region are marked with ```_``` as deafult and therefore ```replace_search = "X"``` and ```replace_with = "_"``` can be used as arguments in ```cleanAnnotation()``` to replace these. This allows for full flexibility of the input data here, depending of the software used to generate determine the peptide sequences. NB: characters other than ```_``` can be used, but these need to be declared when calling buildPWM and scoreSequences functions later (see their help files).

Calling ```cleanAnnotation()``` will produce a new table with the unique combinations of peptide sequences extracted from the annotation column into the peptide column:

```{r echo=TRUE}
annotated_data <- cleanAnnotation(input_data = sample_data, 
                                  annotation_delimiter = "|",
                                  multi_protein_delimiter = ":", 
                                  multi_site_delimiter = ";",
                                  seq_number = 4, 
                                  replace = TRUE, 
                                  replace_search = "X",
                                  replace_with = "_")

head(annotated_data)

```

## Build Position Weight Matrices (PWMs)
The first step to inferring kinase activity, is to build Position Weight Matrices (PWMs) for kinases. This can be done using ```buildPWM()``` for any table containing centered substrate peptide sequences for a list of kinases. The example data ```data(phosphositeplus_human)``` indicates the required format for building PWM models. Below, for demosntration, we use a subset that was sampled above ```sample_pwm```

To generate the PWMs:
```{r echo=TRUE}
pwms <- buildPWM(sample_pwm)
```

This will build the PWM models, accessible as ```PWM$pwm``` and list the number of substrate sequences used to build each PWM, accesible as ```PWM$kinase```.

To view the list of kinases and the number of sequences used:
```{r echo=TRUE}
head(pwms$kinase)
```

### Visualising motifs

Motif amino acids are coloured according to their properties. ```color_scheme``` parameter allows options are "lesk" or "shapely" (default). Y-axis is the information content, measured as bits.

```{r echo=TRUE}
CAMK2A_pwm <- buildPWM(CAMK2A_example)
CAMK2A <- viewPWM(CAMK2A_pwm, 
                  which_pwm="CAMK2A",
                  view_pwm = TRUE,
                  color_scheme = "shapely")
```


## Score PWM matches against peptide sequences

Next, we will use the PWM models generated, ```pwms```, to identify matches in the ```annotated_data``` table that was cleaned using ```cleanAnnotation()``` above. ``scoreSequences``` supports multi-core processing - see the example below for setting the number of workers to 4. ```scoreSequences``` draws a random background by default of size ```n = 1000```. It is recommended to use ```set.seed()``` prior to calling ```scoreSequences``` if you wish to reproduce your results. To access the help file, which explains all the arguments, type ```?scoreSequences``` into the console.

```{r echo=TRUE}
# As an example of control over multi-core processing
# load BiocParallel library
library(BiocParallel)
# finally set/register the number of cores to use
register(SnowParam(workers = 4))

# set seed for reproducible results
set.seed(1234)
scores <- scoreSequences(input_data = annotated_data, 
                         pwm_in = pwms,
                         n = 100)
```

The outputs of ```scores``` are transparent and accessible. These are however primarily intermediate tables for obtaining swing scores. ```scores``` is a simple list object that contains peptide scores ```(scores$peptide_scores)```, p-values for the peptide scores ```(scores$peptide_p)``` and the background peptides used to score significance ```(scores$background)``` for reproducibility (i.e. the background can saved and reused for reproducibility).

In summary, ```scoreSequences()``` scores each input sequence for a match against all PWMs provided using ```buildPWM()`` and generates p-values for scores. This is effectively one large network of kinase-substrate edges of dimensions kinase, ***k***, by substrate, ***s***.

## Predict kinase activity using swing()

Having built a kinase-substrate network, ```swing()``` then integrates the kinase-substrate predictions, directionality and significance of phosphopeptide fold change to assess local connectivity (or swing) of kinase-substrate networks. The final score is a normalised score of predicted kinase activity that is weighted for the number of substrates used in the PWM model and number of peptides in the local kinase-substrate network. By default, this will permute the network 1000 times (here we use 10 for example purposes). It is recommended to use ```set.seed()``` prior to calling ```swing``` if you wish to reproduce your results. ``swing``` supports multi-core processing - see the example below for setting the number of workers to 4.

```{r echo=TRUE}
# after loading BiocParallel library, set/register the number of cores to use
register(SnowParam(workers = 4))

# set seed for reproducible results
set.seed(1234)
swing_out <- swing(input_data = annotated_data, 
                   pwm_in = pwms, 
                   pwm_scores = scores,
                   permutations = 10)

# This will produce two tables, one is a network for use with e.g. Cytoscape and the other is the scores. To access the scores:
head(swing_out$scores)
```

The outputs of this table indicate the following:

+ ```kinase```: The kinase
+ ```pos```: Number of ***positively*** regulated kinase substrates
+ ```neg```: Number of ***negatively*** regulated kinase substrates
+ ```all```: Total number of regulated kinase substrates
+ ```pk```: Proportion of ***positively*** regulated kinase substrates
+ ```nk```: Proportion of ***negatively*** regulated kinase substrates
+ ```swing_raw```: Raw - weighted score
+ ```n```:  Number of subtrate sequence in ```kinase``` PWM
+ ```swing```: Normalised (Z-score transformed) - weighted score
+ ```p_greater```: probability of observing a swing score greater than
+ ```p_less```:  probability of observing a swing score less than

Note that the ``pos``, ``neg`` and ``all`` include a pseudo-count, that is set in ``?swing``, note ``pseudo_count``.

*** See @Kasper2019 for methods description ***

# KinSwingR algorithm

*** For a full description of the KinSwing algorithm, see @Kasper2019 ***

**In brief:**

```buildPWM()``` generates Position Weight Matrices (PWMs) for kinases based on known substrate sequence (Equation 1), where each kinase, $K$, is considered as the log-likelihood ratio of the average frequency of amino acid, $a$, at each position, $p$, divided by background frequencies, $B$ ($C$ is a pseudo count to avoid log zero):

***Equation 1:*** $PWM_{(a,p)}=log((1/n∑^n_{i=1}K_i)+C)/B_a+C)$

```scoreSequences()``` scores each kinase, $K$, match to a substrate $S$, given as $S_{score}=∑^n_{(i=1)}f(a,p)$ , which corresponds to the sum of the corresponding amino acid, $a$, of peptide sequence length, $i$, from position, $p$, of $PWM_{(a,p)}$ and $f(a,p)=PWM_{ap}∈PWM_{(a,p)}$. The probability of observing $S_score$ for kinase, $K$, is determined as conditional on a randomly sampled reference distribution of size $N$ sequences $P(S_{score}|R,N)$, where $R$ sequences are determined to have a test statistic less than or equal to $S_{score}$:

***Equation 2:*** $R= ∑^N_{n=1}I((S_{score})n* ≥ (S_{score})i)$

```swing()``` integrates phosphosite data and kinase-substrate scores from ```scoreSequences()``` into a network for scoring kinase activity based on local connectivity, $swing_k$, (Equation 3). $swing_k$ is the weighted product of the proportion of positive, $Pos_k$, and negative, $Neg_k$, network edges, determined as the product of a logic function (described here: @Kasper2019) given a local network of size, $C_k$, with $n$ substrates for kinase, $K$:

***Equation 3:*** $swing_k=log_2((Pos_k+c)/(Neg_k+c))*log_2(C_k)*log_2(S_n)$

$swing_k$, is transformed into a z-score, $Z(swing_k)$, where, $μ$, is the mean and, $σ$, the standard deviation of swing scores, thus allowing for comparison of predicted kinase activity across multiple timepoints and/or conditions. 

KinSwingR addresses the question of “how likely is it is observe the predicted activity of kinase, $K$, by random chance?” by computing $swing_k$ given $N$ permutations of kinase node labels, $K$, to substrates, $S$, of the total network, $M_{ks}$. Thus, the probability of observing $swing_k$ is conditional on this permuted reference distribution, of size, $N$ (Equation 2). This is computed for each tail of the distribution, that is, positive and negative $swing_k$ scores.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_seqs.r
\name{cleanAnnotation}
\alias{cleanAnnotation}
\title{Function for extracting peptide sequences from multimapped or complex 
annotated data}
\usage{
cleanAnnotation(input_data = NULL, annotation_delimiter = "|",
  multi_protein_delimiter = ":", multi_site_delimiter = ";",
  seq_number = 4, replace = FALSE, replace_search = "X",
  replace_with = "_", verbose = FALSE)
}
\arguments{
\item{input_data}{A data.frame of phosphopeptide data. Must contain 4 columns
and the following format must be adhered to. Column 1 - Annotation, Column 2
 - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
 - p-value [0-1]. This will extract the peptide sequences from Column1 and 
 replace all values in Column2 to be used in scoreSequences(). Where 
 peptide sequences have not been extracted from the annotation, leave 
 Column2 as NA's.}

\item{annotation_delimiter}{The character used to delimit annotations. 
Default="|"}

\item{multi_protein_delimiter}{The character used to delimit multi-protein 
assignments. Default=":". E.g. Ddx17:Ddx2}

\item{multi_site_delimiter}{The character used to delimit multi-site 
assignments. Default=";". E.g. 494;492}

\item{seq_number}{The annotation frame that contains the sequence after 
delimitation. E.g. The sequence "RSRYRTTSSANNPN" is contained in the 4th 
annotation frame of the following annotation: 
"A0A096MIX2|Ddx17|494|RSRYRTTSSANNPN" and would therefore set seq_number=4.
Default=4}

\item{replace}{Replace a letter that describes sequences outside of the 
protein after centering on the phosphosite (e.g X in XXXMERSTRELCLNF). Use in
 combination with replace_search and replace_with to replace amino acids. 
 Options are "TRUE" or "FALSE". Default="FALSE".}

\item{replace_search}{Amino Acid to search for when replacing sequences. 
Default="X"}

\item{replace_with}{Amino Acid to replace with when replacing sequences. 
Default="_"}

\item{verbose}{Print progress to screen. Default=FALSE}
}
\value{
A data.table with the peptides extracted from the annotation column
}
\description{
This function extracts unique peptide:annotation combinations 
from complex annotated data and formats for further analysis using KinSwingR.
For instance, example input annotation may be: 
"A0A096MIX2|Ddx17|494|RSRYRTTSSANNPN". This function will extract the peptide
 sequence into a second column and associate it all annotations. See vignette
  for more details.
}
\examples{
## Extract peptide sequences from annotation data:
data(example_phosphoproteome)

## A0A096MJ61|NA|89|PRRVRNLSAVLAART
## The following will extract all the uniquely annotated peptide
## sequences from the "annotation" column and place these in the
## "peptide" column. Where multi-mapped peptide sequences are input,
## these are placed on a new line.
##
## Here, sequences with a "X" and also replaced with a "_". This is ensure 
## that PWMs are built correctly.

## Sample data for demonstration:
sample_data <- head(example_phosphoproteome)
annotated_data <- cleanAnnotation(input_data = sample_data,
                                   annotation_delimiter = "|",
                                   multi_protein_delimiter = ":",
                                   multi_site_delimiter = ";",
                                   seq_number = 4,
                                   replace = TRUE,
                                   replace_search = "X",
                                   replace_with = "_")

## Return the annotated data with extracted peptides:
head(annotated_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_pwms.r
\name{buildPWM}
\alias{buildPWM}
\title{Generate Position Weight Matrices (PWMs)}
\usage{
buildPWM(kinase_table = NULL, wild_card = "_", substrate_length = 15,
  substrates_n = 10, pseudo = 0.01, remove_center = FALSE,
  verbose = FALSE)
}
\arguments{
\item{kinase_table}{A data.frame of substrate sequences and kinase names. 
Format of data must be as follows: column 1 - kinase/kinase family 
name/GeneID, column 2 - centered peptide seqeuence.}

\item{wild_card}{Letter to describe sequences that are outside of the protein
after centering on the phosphosite (e.g ___MERSTRELCLNF). Default: "_".}

\item{substrate_length}{Full length of substrate sequence (default is 15). 
Will be trimmed automatically or report error if sequences in kinase_table 
are not long enough.}

\item{substrates_n}{Number of sequences used to build a PWM model. Low 
sequence counts will produce poor representative PWM models. Default: "10"}

\item{pseudo}{Small number to add to values for PWM log transformation to 
prevent log transformation of zero. Default = 0.01}

\item{remove_center}{Remove all peptide seqeuences with the central amino 
acid matching a character (e.g. "y"). Default = FALSE}

\item{verbose}{Print progress to screen. Default=FALSE}
}
\value{
Output is a list containing two tables, "pwm" and "kinase". To access
 PWMs: pwms$pwm and Table of Kinase and sequence counts: 
 pwms$kinase
}
\description{
Generate Position Weight Matrices (PWMs) for a table containing 
centered substrate peptide sequences for a list of kinases. The output of 
this function is to be used for scoring PWM matches to peptides via 
scoreSequences()
}
\examples{
## Build PWM models from phosphositeplus data with default of minimum
## of 10 substrate sequences for building a PWM model.

data(phosphositeplus_human)

##randomly sample 1000 substrates for demonstration.
set.seed(1)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
1000),]
pwms <- buildPWM(sample_pwm)

## Data frame of models built and number of sequences used to build each
## PWM model:
head(pwms$kinase)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_phosphoproteome.R
\docType{data}
\name{example_phosphoproteome}
\alias{example_phosphoproteome}
\title{Example phosphoproteome.}
\format{A data frame with 6215 rows and 4 variables:
\describe{
  \item{annotation}{Annotation of phosphorylated peptides}
  \item{peptide}{blank - peptides need to be extracted from
  annotation}
  \item{fc}{Fold Change (log2)}
  \item{pval}{P-value for fold-change.}
}}
\usage{
example_phosphoproteome
}
\description{
A dataset containing annotated subtrate sequences derived
from XXX. See original publication for more
details: Engholm-Keller & Waardenberg AJ et al.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view_pwm.r
\name{viewPWM}
\alias{viewPWM}
\title{View motif}
\usage{
viewPWM(pwm_in = NULL, which_pwm = NULL, fontsize = 10,
  view_pwm = FALSE, pseudo = 0.01, convert_PWM = FALSE,
  color_scheme = "shapely", correction_factor = NULL)
}
\arguments{
\item{pwm_in}{View a PWM provided using the buildPWM. Default = NULL}

\item{which_pwm}{If pwms are input (outputs of buildPWM), a kinase name must
match a name in pwms$kinase$kinase list of names. Default = NULL}

\item{fontsize}{Font size to use on x and y axis. Default = 10}

\item{view_pwm}{View the PWM. Default = FALSE}

\item{pseudo}{Small amount added to the PWM model, where zero's exist, to 
avoid log zero. Default = 0.01}

\item{convert_PWM}{pwm_in is a matrix of counts at position. TRUE will 
convert this matrix to a PWM. Default = FALSE}

\item{color_scheme}{Which color scheme to use for Amino Acid Groups. Options
are "lesk" or "shapely". Default = "shapely"}

\item{correction_factor}{Number of sequences used to infer the PWM. 
This can be used where a small number of sequences were used to build the 
model and included as E_n in the Shannon's Entropy Model. Default = NULL}
}
\value{
Visualisation of a motif, scaled on bits and two tables. 1) pwm: 
corresponding to the PWM from pwm and 2) pwm_bits: corresponding to the
conversion to bits.
}
\description{
View information content for each position of the PWM.
Information content is modelled using Shannon's Entropy Model. The maximum
information content is therefore log2(n), where n is the number of amino
acids. Colors of Amino Acids are in accordance with the Lesk scheme.
}
\examples{
## Build PWM models from phosphositeplus data with default of minimum
## of 10 substrate sequences for building a PWM model.
data(phosphositeplus_human)
##randomly sample 1000 substrates for demonstration.
set.seed(1)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
1000),]
pwms <- buildPWM(sample_pwm)

## Data frame of models built and number of sequences used to build each
## PWM model:
head(pwms$kinase)
## Will not visualise the motif
CAMK2A_motif <- viewPWM(pwm_in = pwms, 
                        which_pwm = "CAMK2A",
                        view_pwm = FALSE)
# Use view_pwm = TRUE to view the motif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swing.r
\name{swing}
\alias{swing}
\title{Swing statistic}
\usage{
swing(input_data = NULL, pwm_in = NULL, pwm_scores = NULL,
  pseudo_count = 1, p_cut_pwm = 0.05, p_cut_fc = 0.05,
  permutations = 1000, return_network = FALSE, verbose = FALSE)
}
\arguments{
\item{input_data}{A data.frame of phoshopeptide data. Must contain 4 columns
and the following format must be adhered to. Column 1 - Annotation, Column 2
 - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
 - p-value [0-1]. This must be the same dataframe used in scoreSequences()}

\item{pwm_in}{List of PWMs created using buildPWM()}

\item{pwm_scores}{List of PWM-substrate scores created using
scoreSequences()}

\item{pseudo_count}{Pseudo-count acts at two levels. 1) It adds a small
number to the counts to avoid zero divisions, which also 2) avoids log-zero 
transformations. Note that this means that pos, neg and all values in the
output table include the addition of the pseudo-count. Default: "1"}

\item{p_cut_pwm}{Significance level for determining a significant
kinase-substrate enrichment. Default: "0.05"}

\item{p_cut_fc}{Significance level for determining a significant level of
Fold-change in the phosphoproteomics data. Default: "0.05"}

\item{permutations}{Number of permutations to perform. This will shuffle the 
kinase-subtrate edges of the network n times. To not perform permutations and
 only generate the scores, set permutations=1 or permutations=FALSE. Default:
  "1000"}

\item{return_network}{Option to return an interaction network for visualising
in cystoscape. Default = FALSE}

\item{verbose}{Turn verbosity on/off. To turn on, verbose=TRUE. Options are: 
"TRUE, FALSE". Default=FALSE}
}
\value{
A data.table of swing scores
}
\description{
This function integrates the kinase-substrate predictions,
directionality of phosphopeptide fold change and signficance to assess
local connectivity (swing) of kinase-substrate networks. The final score
is a normalised and weighted score of predicted kinase activity. If 
permutations are selected, network node:edges are permutated. P-values will 
be calculated for both ends of the distribution of swing scores (positive and
 negative swing scores).
}
\examples{
## import data
data(example_phosphoproteome)
data(phosphositeplus_human)

## clean up the annotations
## sample 100 data points for demonstration
sample_data <- head(example_phosphoproteome, 100)
annotated_data <- cleanAnnotation(input_data = sample_data)

## build the PWM models:
set.seed(1234)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
1000),]
pwms <- buildPWM(sample_pwm)

## score the PWM - substrate matches
## Using a "random" background, to calculate the p-value of the matches
## Using n = 100 for demonstration
## set.seed for reproducibility
set.seed(1234)
substrate_scores <- scoreSequences(input_data = annotated_data,
                                   pwm_in = pwms,
                                   background = "random",
                                   n = 100)

## Use substrate_scores and annotated_data data to predict kinase activity.
## This will permute the network node and edges 10 times for demonstration.
## set.seed for reproducibility
set.seed(1234)
swing_output <- swing(input_data = annotated_data,
                      pwm_in = pwms,
                      pwm_scores = substrate_scores,
                      permutations = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phosphositeplus_human.R
\docType{data}
\name{phosphositeplus_human}
\alias{phosphositeplus_human}
\title{Human kinase-substrates derived from PhosphositePlus.}
\format{A data frame with 11985 rows and 2 variables:
\describe{
  \item{kinase}{human kinase gene symbol}
  \item{substrate}{centered substrate sequence for kinase}
}}
\source{
\url{https://www.phosphosite.org/}
}
\usage{
phosphositeplus_human
}
\description{
A dataset containing human kinases and subtrate sequences. See
original publication for more details: Hornbeck et al.
Nucleic Acids Res. 40:D261-70, 2012
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kinswing.r
\docType{package}
\name{KinSwingR}
\alias{KinSwingR}
\alias{KinSwingR-package}
\title{KinSwingR: A package for predicting kinase activity}
\description{
This package provides functionality for kinase-subtrate prediction, and 
integration with phosphopeptide fold change and signficance to assess the 
local connectivity (swing) of kinase-substrate networks. The final output of 
KinSwingR is a score that is normalised and weighted for prediction of kinase
 activity.
}
\details{
Contact a.waardenberg@gmail.com for questions relating to functionality.
}
\section{buildPWM function}{

Builds PWMs for kinases from a table of kinases and known substrate 
sequences.
}

\section{scoreSequences function}{

Score kinase PWMs matches against a set of peptide seqeuences.
}

\section{swing function}{

Integrates kinase PWMs matches against peptide seqeuences and directionality 
as well as significance of peptides for prediction of kinase activity.
}

\section{cleanAnnotation function}{

Function for extracting peptides from multimapped data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_sequences.r
\name{scoreSequences}
\alias{scoreSequences}
\title{Score substrate sequences for matches to kinase Position Weight Matrices 
(PWMs)}
\usage{
scoreSequences(input_data = NULL, pwm_in = NULL,
  background = "random", n = 1000, force_trim = FALSE,
  verbose = FALSE)
}
\arguments{
\item{input_data}{A data.frame of phoshopeptide data. Must contain 4 columns
and the following format must be adhered to. Column 1 - Annotation, Column 2
 - centered peptide sequence, Column 3 - Fold Change [-ve to +ve], Column 4 
 - p-value [0-1]}

\item{pwm_in}{List of PWMs created using buildPWM()}

\item{background}{Option to provide a data.frame of peptides to use as 
background. If providing a background as a table, this must contain two 
columns; Column 1 - Annotation, Column 2 - centered peptide sequence. These 
must be centered. OR generate a random background for PWM scoring from the
input list - background = random. Default: "random"}

\item{n}{Number of permutations to perform for generating background. 
Default: "1000"}

\item{force_trim}{This function will detect if a peptide sequence is of 
different length to the PWM models generated (provided in pwm_in) and trim 
the input sequences to the same length as the PWM models. If a background is 
provided, this will also be trimmed to the same width as the PWM models. 
Options are: "TRUE, FALSE". Default = FALSE}

\item{verbose}{Turn verbosity on/off. To turn on, verbose=TRUE. Options are: 
"TRUE, FALSE". Default = FALSE}
}
\value{
A list with 3 elements: 1) PWM-substrate scores: 
substrate_scores$peptide_scores, 2) PWM-substrate p-values: 
substrate_scores$peptide_p 3) Background used for reproducibility: 
substrate_scores$background 4) input_data is returned in the case that it was
trimmed.
}
\description{
Scores each input sequence for a match against all PWMs provided
 from buildPWM() and generates p-values for scores. The output of this 
 function is to be used for building the swing metric, the predicted activity
  of kinases.
}
\examples{
## import data
data(example_phosphoproteome)
data(phosphositeplus_human)

## clean up the annotations
## sample 100 data points for demonstration
sample_data <- head(example_phosphoproteome, 100)
annotated_data <- cleanAnnotation(input_data = sample_data)

## build the PWM models:
set.seed(1234)
sample_pwm <- phosphositeplus_human[sample(nrow(phosphositeplus_human), 
1000),]
pwms <- buildPWM(sample_pwm)

## score the PWM - substrate matches
## Using a "random" background, to calculate the p-value of the matches
## Using n=10 for demonstration
## set.seed for reproducibility
set.seed(1234)
substrate_scores <- scoreSequences(input_data = annotated_data,
                                   pwm_in = pwms,
                                   background = "random",
                                   n = 10)

}
